/*
 *  net/sf/logdistiller/Condition.java
 *  Copyright (C) 2004-2007 Herv BOUTEMY
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package net.sf.logdistiller;

import java.io.Serializable;

import org.apache.commons.lang.ArrayUtils;
import org.jdom.Element;

/**
 * One condition for a LogEvent to be selected in a group. It consists of
 * matches on attributes of the LogEvent.
 */
public class Condition implements Serializable {
  private static final long serialVersionUID = 9085774334169039381L;
  private final Match[] matchs;

  public Condition(Match[] matchs) {
    this.matchs = (Match[])ArrayUtils.clone(matchs);
  }

  public boolean match(LogEvent le) {
    int count = matchs.length;
    for(int i = 0; i < count; i++) {
      if (! matchs[i].match(le)) {
        return false;
      }
    }
    return true;
  }

  public Element dump() {
    Element elmt = new Element("condition");
    int count = matchs.length;
    for(int i = 0; i < count; i++) {
      elmt.addContent(matchs[i].dump());
    }
    return elmt;
  }
}
