/*
 *  net/sf/logdistiller/PublishHelper.java
 *  Copyright (C) 2004-2007 Herv BOUTEMY
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package net.sf.logdistiller;

import java.io.*;
import java.util.*;

public class PublishHelper {
  private final LogDistillation ld;
  private final List publishers;

  public PublishHelper(LogDistillation ld) {
    this(ld, new ArrayList());
  }

  public PublishHelper(LogDistillation ld, List publishers) {
    this.ld = ld;
    this.publishers = publishers;
  }

  public int publish() throws IOException, Publisher.PublishException {
    int published = publishGlobal();

    LogDistillation.Group[] groups = ld.getGroups();
    int count = groups.length;
    for (int i = 0; i < count; i++) {
      LogDistillation.Group group = groups[i];
      if ((! group.isSkip()) && (group.getEventCount() > 0)) {
        published += publishGroup(group);
      }
    }
    return published;
  }

  public int publishGlobal() throws IOException, Publisher.PublishException {
    LogDistiller.Report[] reports = ld.getDefinition().getOutput().getReports();
    int count = reports.length;
    int published = 0;
    for (int i = 0; i < count; i++) {
      LogDistiller.Report report = reports[i];
      Publisher publisher = Publishers.getPublisher(report.getPublisher());
      if (isPublisherEnabled(publisher)) {
        publisher.publish(ld, report);
        published++;
      }
    }
    return published;
  }

  public int publishGroup(LogDistillation.Group group) throws IOException, Publisher.PublishException {
    LogDistiller.Report[] reports = group.getDefinition().getReports();
    int count = reports.length;
    int published = 0;
    for (int i = 0; i < count; i++) {
      LogDistiller.Report report = reports[i];
      Publisher publisher = Publishers.getPublisher(report.getPublisher());
      if (isPublisherEnabled(publisher)) {
        publisher.publish(group, report);
        published++;
      }
    }
    return published;
  }

  private boolean isPublisherEnabled(Publisher publisher) {
    return (publisher != null) && ((publishers == null) || publishers.contains(publisher.getId()));
  }
}
