/*
 *  net/sf/logdistiller/Publisher.java
 *  Copyright (C) 2004-2007 Herv BOUTEMY
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package net.sf.logdistiller;

import java.io.IOException;

/**
 * Base class of report publishers, for both global and group reports.
 */
public abstract class Publisher {
  /**
   * Get the publisher-id for this class.
   * @return a unique publisher id
   */
  public abstract String getId();

  /**
   * Publish a global report.
   * @param logdistillation the logdistillation containing the results to publish
   * @param report the report configuration
   * @throws IOException
   * @throws PublishException
   */
  public abstract void publish(LogDistillation logdistillation, LogDistiller.Report report)
      throws IOException, PublishException;

  /**
   * Publish a group report.
   * @param group the logdstillation group containing the results to publish
   * @param report the report configuration
   * @throws IOException
   * @throws PublishException
   */
  public abstract void publish(LogDistillation.Group group, LogDistiller.Report report)
      throws IOException, PublishException;

  public static class PublishException extends Exception {
    private static final long serialVersionUID = 498326805296616769L;

    public PublishException(String message, Throwable nested) {
      super(message, nested);
    }
  }
}
