/*
 *  net/sf/logdistiller/ReportFormat.java
 *  Copyright (C) 2004-2007 Herv BOUTEMY
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package net.sf.logdistiller;

import java.io.IOException;
import java.io.Writer;

/**
 * Report formats' base class.
 */
public abstract class ReportFormat {
  private final String id;

  protected ReportFormat(String id) {
    this.id = id;
  }

  public String getId() {
    return id;
  }

  public abstract String getContentType();

  public abstract String getFileExtension();

  public abstract void report(LogDistillation ld, Writer output) throws IOException;

  public abstract void report(LogDistillation.Group group, Writer output) throws IOException;

  protected int addPluginsGroupReport(LogDistillation.Group group, PluginReport report) {
    LogDistillation.Plugin[] plugins = group.getPlugins();
    int len = plugins.length;
    int count = 0;
    for (int i = 0; i < len; i++) {
      LogDistillation.Plugin plugin = plugins[i];
      if (plugin.getDefinition().isGroupReportEnabled()) {
        plugin.appendGroupReport(report);
        count++;
      }
    }
    return count;
  }

  protected int addPluginsGlobalReport(LogDistillation.Group group, PluginReport report) {
    LogDistillation.Plugin[] plugins = group.getPlugins();
    int len = plugins.length;
    int count = 0;
    for (int i = 0; i < len; i++) {
      LogDistillation.Plugin plugin = plugins[i];
      if (plugin.getDefinition().isGlobalReportEnabled()) {
        plugin.appendGlobalReport(report);
        count++;
      }
    }
    return count;
  }

  public static abstract class PluginReport {
    public abstract void beginPluginReport(LogDistillation.Plugin plugin, String description);
    public abstract void addLink(String filename, String description);
    public abstract void addParam(String param, String value);
    public abstract void addItem(int count, String description);
    public abstract void endPluginReport();
  }
}
