/*
 *  net/sf/logdistiller/ReportFormats.java
 *  Copyright (C) 2004-2007 Herv BOUTEMY
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package net.sf.logdistiller;

import java.util.*;

import net.sf.logdistiller.reports.BaseReportFormats;
import net.sf.logdistiller.util.ExtensionHelper;
import net.sf.logdistiller.util.FormatUtil;

/**
 * Definition of available report formats. By default, some report formats come
 * {@link net.sf.logdistiller.reports.BaseReportFormats predefined} with <b>LogDistiller</b>, but you can add your own.
 */
public abstract class ReportFormats {
  public abstract List defineReportFormats();

  public static final List ALL;
  public static final Map MAP;
  static {
    List all = new ArrayList(new BaseReportFormats().defineReportFormats());
    all.addAll(loadExtensionReportFormats());
    ALL = Collections.unmodifiableList(all);

    Map map = new HashMap();
    Iterator iter = ALL.iterator();
    while (iter.hasNext()) {
      ReportFormat reportFormat = (ReportFormat)iter.next();
      map.put(reportFormat.getId(), reportFormat);
    }
    MAP = Collections.unmodifiableMap(map);
  }

  /**
   * get all the plugins defined (predefined and custom).
   */
  public static List getAllReportFormats() {
    return ALL;
  }

  public static String listAllReportFormatIds() {
    return FormatUtil.join(", ", MAP.keySet().iterator());
  }

  public static ReportFormat getReportFormat(String id) {
    ReportFormat format = (ReportFormat)MAP.get(id);
    if (format == null) {
      throw new IllegalArgumentException("unknown report format '" + id + "', known values are: " + listAllReportFormatIds());
    }
    return format;
  }

  private static List loadExtensionReportFormats() {
    Iterator iter = ExtensionHelper.findExtensions("reportformats").iterator();
    List reportformats = new ArrayList();
    while (iter.hasNext()) {
      String reportformatsClass = (String)iter.next();
      if (reportformatsClass != null) {
        reportformats.addAll(loadReportFormats(reportformatsClass));
      }
    }
    return reportformats;
  }

  private static List loadReportFormats(String reportFormatsClass) {
    try {
      ReportFormats reportFormats = (ReportFormats) Class.forName(reportFormatsClass).newInstance();
      return reportFormats.defineReportFormats();
    } catch (ClassNotFoundException cnfe) {
      throw new RuntimeException("unable to load report formats class " + reportFormatsClass, cnfe);
    } catch (IllegalAccessException iae) {
      throw new RuntimeException("unable to access report formats constructor for class " + reportFormatsClass, iae);
    } catch (InstantiationException ie) {
      throw new RuntimeException("unable to instanciate report formats class " + reportFormatsClass, ie);
    }
  }
}
