/*
 *  net/sf/logdistiller/ant/AntLogEvent.java
 *  Copyright (C) 2004-2007 Herv BOUTEMY
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package net.sf.logdistiller.ant;

import org.apache.tools.ant.Project;
import org.apache.tools.ant.types.Path;
import org.apache.tools.ant.types.Reference;

/**
 * Definition of logevent class in Ant buildfile.
 * <pre>&lt;logevent factory="<i>[event factory class]</i>"/&gt;</pre>
 */
public class AntLogEvent {
  private final Project project;
  private String logtype;
  private Path classpath;

  public AntLogEvent(Project project) {
    this.project = project;
  }

  public void setClasspath(Path classpath) {
    if (this.classpath == null) {
      this.classpath = classpath;
    } else {
      this.classpath.append(classpath);
    }
  }

  public Path createClasspath() {
    if (classpath == null) {
      classpath = new Path(project);
    }
    return classpath.createPath();
  }

  public void setClasspathRef(Reference r) {
    createClasspath().setRefid(r);
  }

  public void setLoaderRef(Reference r) {
  }

  /**
   * @deprecated replaced by <code>logtype</code> attribute
   */
  public void setFactory(String factory) {
    setLogtype(factory);
    project.log("WARNING: attribute 'factory' is deprecated, please use attribute 'logtype' instead",
                Project.MSG_WARN);
  }

  public void setLogtype(String logtype) {
    this.logtype = logtype;
  }

  public String getLogtype() {
    return logtype;
  }
}
