/*
 *  net/sf/logdistiller/gui/MainPanel.java
 *  Copyright (C) 2004-2007 Herv BOUTEMY
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package net.sf.logdistiller.gui;

import java.awt.Font;
import java.awt.LayoutManager;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.io.*;
import java.lang.reflect.*;
import java.text.MessageFormat;
import java.text.ParseException;
import java.util.*;
import javax.swing.*;
import javax.xml.parsers.ParserConfigurationException;

import org.xml.sax.ErrorHandler;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

import com.jgoodies.forms.builder.DefaultFormBuilder;
import com.jgoodies.forms.factories.ButtonBarFactory;
import com.jgoodies.forms.factories.DefaultComponentFactory;
import com.jgoodies.forms.layout.FormLayout;
import com.sun.syndication.io.XmlReader;

import org.apache.commons.io.IOUtils;

import net.sf.logdistiller.LogDistillation;
import net.sf.logdistiller.LogDistiller;
import net.sf.logdistiller.LogEvent;
import net.sf.logdistiller.LogType;
import net.sf.logdistiller.LogTypes;
import net.sf.logdistiller.reports.TextReport;
import net.sf.logdistiller.util.FormatUtil;
import net.sf.logdistiller.util.PropertiesReplacer;
import net.sf.logdistiller.xml.DOMConfigurator;
import net.sf.logdistiller.xml.LogDistillerEntityResolver;

/**
 * Main gui panel.
 */
public class MainPanel implements Runnable {
  private static final String TITLE = "LogDistiller " + LogDistiller.getVersion() + " interactive console";
  private static final String ABOUT = TITLE
      + "\n\nCopyright (C) 2004-2007 Herv Boutemy. All rights reserved.";
  
  private final JFrame frame;
  public MainPanel(JFrame frame) {
    this.frame = frame;
    frame.setTitle(TITLE);
  }

  private Thread reloadThread;
  protected JPanel panel = new JPanel();
  private JTextField tfRules = new JTextField("open or create a rules configuration file...");
  private JTextField tfSourceFile = new JTextField();
  private JTextField tfSourceStats = new JTextField();
  private JTextField tfOutputDir = new JTextField();
  private JButton btRun = new JButton("run");
  private JButton btBatch = new JButton("batch");
  private JButton btSourceFile = new JButton("...");
  private JButton btOutputDir = new JButton("...");
  private JTextArea taResult = new JTextArea();
  private LongProgressBar progress = new LongProgressBar();

  private void initComponents() {
    tfSourceFile.setEditable(false);
    btSourceFile.addActionListener(SwingAdapter.getActionListener(this, "btSourceFileActionPerformed"));
    tfSourceStats.setEditable(false);
    tfOutputDir.setEditable(false);
    btOutputDir.addActionListener(SwingAdapter.getActionListener(this, "btOutputDirActionPerformed"));
    tfRules.setEditable(false);
    taResult.setEditable(false);
    btRun.addActionListener(SwingAdapter.getActionListener(this, "btRunActionPerformed"));
    btRun.setEnabled(false);
    btBatch.addActionListener(SwingAdapter.getActionListener(this, "btBatchActionPerformed"));
    btBatch.setEnabled(false);

    taResult.setFont(new Font("Monospaced", Font.PLAIN, taResult.getFont().getSize()));

    progress.setStringPainted(true);
    progress.setFont(new Font("Monospaced", Font.PLAIN, progress.getFont().getSize()));
  }

  private void addLabel(String label, String id) {
    panel.add(new JLabel(label), id);
  }

  private void addSeparator(String label, String id) {
    panel.add(DefaultComponentFactory.getInstance().createSeparator(label), id);
  }

  public final static SimpleLayoutConstraintsManager SIMPLE_LAYOUT_CONSTRAINTS_MANAGER =
      SimpleLayoutConstraintsManager.getLayoutConstraintsManager(MainPanel.class.getResourceAsStream("gui.xml"));

  public LayoutManager createLayout(String layoutId) {
    return MainPanel.SIMPLE_LAYOUT_CONSTRAINTS_MANAGER.getLayout(layoutId);
  }

  public JComponent buildPanel() {
    initComponents();
    buildMenuBar();

    panel.setBorder(com.jgoodies.forms.factories.Borders.DIALOG_BORDER);
    panel.setLayout(createLayout("mainPanel"));

    /*org.mlc.swing.layout.LayoutConstraintsManager layoutConstraintsManager =
      org.mlc.swing.layout.LayoutConstraintsManager.getLayoutConstraintsManager(MainPanel.class.getResourceAsStream("gui.xml"));
    panel.setLayout(layoutConstraintsManager.createLayout("mainPanel", panel));
    new org.mlc.swing.layout.LayoutFrame(layoutConstraintsManager).setVisible(true);*/

    panel.add(tfRules, "tfRules");

    addSeparator("logs", "sep2");
    addLabel("log file:", "lblSourceFile");
    panel.add(tfSourceFile, "tfSourceFile");
    panel.add(btSourceFile, "btSourceFile");
    panel.add(tfSourceStats, "tfSourceStats");

    addLabel("output directory:", "lblOutputDir");
    panel.add(tfOutputDir, "tfOutputDir");
    panel.add(btOutputDir, "btOutputDir");

    JPanel bbar = ButtonBarFactory.buildRightAlignedBar(new JButton[] { btRun, btBatch });
    FormLayout layout = new FormLayout("default:grow, 3dlu, pref");
    DefaultFormBuilder builder = new DefaultFormBuilder(layout);
    builder.append(progress, bbar);
    panel.add(builder.getPanel(), "pRun");

    addSeparator("result", "sep3");
    panel.add(new JScrollPane(taResult), "taResult");

    reloadThread = new Thread(this);
    reloadThread.setDaemon(true);
    reloadThread.setPriority(Thread.MIN_PRIORITY);
    reloadThread.start();
    return panel;
  }
  
  private void buildMenuBar() {
    JMenuBar menuBar = new JMenuBar();
    // FILE MENU
    JMenu fileMenu = new JMenu("File");
    fileMenu.setMnemonic(KeyEvent.VK_F);

    // Open file
    JMenuItem fileOpenMenuItem = new JMenuItem("Open rules file...", KeyEvent.VK_O);
    fileOpenMenuItem.addActionListener(SwingAdapter.getActionListener(this, "btOpenActionPerformed"));
    fileMenu.add(fileOpenMenuItem);
    
    // New file
    JMenuItem fileNewMenuItem = new JMenuItem("New rules file...", KeyEvent.VK_N);
    fileNewMenuItem.addActionListener(SwingAdapter.getActionListener(this, "btNewActionPerformed"));
    fileMenu.add(fileNewMenuItem);

    // exit
    JMenuItem exitMenuItem = new JMenuItem("Exit", KeyEvent.VK_X);
    exitMenuItem.addActionListener(SwingAdapter.getActionListener(this, "exit"));
    fileMenu.add(exitMenuItem);

    // HELP MENU
    JMenu helpMenu = new JMenu("Help");
    helpMenu.setMnemonic(KeyEvent.VK_H);

    // About
    JMenuItem aboutMenuItem = new JMenuItem("About...", KeyEvent.VK_A);
    aboutMenuItem.addActionListener(SwingAdapter.getActionListener(this, "about"));
    helpMenu.add(aboutMenuItem);

    menuBar.add(fileMenu);
    menuBar.add(helpMenu);
    frame.setJMenuBar(menuBar);
  }

  public void about(ActionEvent ae) {
    JOptionPane.showMessageDialog(frame, ABOUT, "About", JOptionPane.INFORMATION_MESSAGE);
  }

  public void exit(ActionEvent ae) {
    System.exit(0);
  }

  private File currentDir = new File(".");
  private File configurationFile = null;
  private long configurationFileLastModified;
  public void btOpenActionPerformed(ActionEvent ae) {
    reloadConfigurationFileEnabled = false;
    JFileChooser fc = new JFileChooser(currentDir);
    int ret = fc.showOpenDialog(frame);
    if (ret == JFileChooser.APPROVE_OPTION) {
      selectConfigurationFile(fc.getSelectedFile());
    }
    reloadConfigurationFileEnabled = true;
  }

  private LogDistiller ld = null;
  public void selectConfigurationFile(File newConfiguration) {
    initProgress();
    configurationFile = newConfiguration;
    configurationFileLastModified = configurationFile.lastModified();
    currentDir = configurationFile.getParentFile();
    frame.setTitle(TITLE + ": " + configurationFile.getName());

    tfRules.setText("reading " + configurationFile.getName());
    try {
      taResult.setText(IOUtils.toString(new XmlReader(configurationFile)));
      DOMConfigurator domConfigurator = new DOMConfigurator(new Properties());
      ld = domConfigurator.read(configurationFile, new GuiSAXErrorHandler());
      compatibilityUpdate(ld);

      LogDistiller.LogType logtype = ld.getLogType();
      String logeventType = (logtype != null) ? logtype.getId() : null;

      // select logevent type and description according to file content
      if (logeventType != null) {
        if (LogTypes.getLogType(logeventType) == null) {
          taResult.setText("Unknown '" + logeventType + "' logtype.\n" +
              "Valid values: " + LogTypes.listAllLogTypeIds());
          logeventType += " (unknown)";
        }
      } else {
        logeventType = "undefined";
        taResult.setText("Undefined logtype.\n" +
            "Valid values: " + LogTypes.listAllLogTypeIds());
      }
      try {
        logtypeDescription = LogTypes.getLogType(logtype.getId()).newDescription(ld.getLogType().getParams());
        ld.checkLogtypeAttributes(logtypeDescription);
      } catch (Exception e) {
        displayException(e);
      }

      // set the destination directory
      selectOutputDir(new File(ld.getOutput().getDirectory()));

      // set the rules text to a description of the rules read
      Object[] args = new Object[] { ld.getId(), ld.getDescription(), new Integer(ld.getGroups().length),
          new Integer(ld.getCategories().length), logeventType
      };
      MessageFormat mf =
          new MessageFormat("{0} - {1}: {2,choice,0#no group|1#one group|1<{2,number,integer} groups}"
                            + " {3,choice,0#with no category|1#in one category|1<in {3,number,integer} categories}"
                            + ", logtype: {4}.");
      tfRules.setText(mf.format(args));
    } catch (RuntimeException re) {
      ld = null;
      displayException(re);
    } catch (IOException ioe) {
      ld = null;
      displayException(ioe);
    } catch (ParserConfigurationException pce) {
      ld = null;
      displayException(pce);
    } catch (SAXException se) {
      ld = null;
      displayException(se);
    }
    updateBtRunState();
  }

  /**
   * Make all that's possible to update latest LogDistiller model to maintain compatibility with older versions, since
   * some elements have moved from Ant task to LogDistiller model.
   * @param ld LogDistiller
   */
  private void compatibilityUpdate(LogDistiller ld) {
    if (ld.getLogType() != null) {
      // ok, no compatibility upgrade to consider
      return;
    }
    String logeventType = (String)ld.getOutput().getParams().get("logevent.type");
    if (logeventType != null) {
      String warning = "parameter 'logevent.type' is deprecated, please use element 'logtype' from " +
         LogDistillerEntityResolver.LATEST_DTD + " in your rules file";
      ld.addWarning(warning);
      ld.buildCompatibilityLogType(logeventType);
    }
  }

  private final Runnable runnableReloadConfigurationFile = SwingAdapter.getRunnable(this, "reloadConfigurationFile");
  private boolean reloadConfigurationFileEnabled = true;
  public void reloadConfigurationFile() {
    selectConfigurationFile(configurationFile);
    initProgress();
  }

  public class GuiSAXErrorHandler implements ErrorHandler {
    public void error(SAXParseException spe) {
      taResult.append("Parsing error on line " + spe.getLineNumber() + " and column " + spe.getColumnNumber() + ": " + spe.getMessage() + "\n");
      Exception e = spe.getException();
      if (e != null) {
        displayException(e);
      }
    }

    public void fatalError(SAXParseException spe) {
      error(spe);
    }

    public void warning(SAXParseException spe) {
      taResult.append("Parsing warning on line " + spe.getLineNumber() + " and column " + spe.getColumnNumber() + ": " + spe.getMessage() + "\n");
      Exception e = spe.getException();
      if (e != null) {
        displayException(e);
      }
    }
  }

  private void displayException(Throwable t) {
    StringWriter sw = new StringWriter();
    PrintWriter out = new PrintWriter(sw);
    t.printStackTrace(out);
    out.close();
    taResult.append(sw.toString());
  }

  private LogType.Description logtypeDescription = null;

  public void btNewActionPerformed(ActionEvent ae) {
    NewDialog dlg = new NewDialog(currentDir);
    File newConfiguration = dlg.showNewConfigurationDialog(frame);
    taResult.setText("");
    if (newConfiguration != null) {
      selectConfigurationFile(newConfiguration);
    }
  }

  private File sourceFile = null;
  private long sourceFileLastModified;
  public void btSourceFileActionPerformed(ActionEvent ae) {
    JFileChooser fc = new JFileChooser((sourceFile == null) ? currentDir : sourceFile.getParentFile());
    int ret = fc.showOpenDialog(frame);
    if (ret == JFileChooser.APPROVE_OPTION) {
      selectSourceFile(fc.getSelectedFile());
    }
  }

  public void selectSourceFile(File newSourceFile) {
    sourceFile = newSourceFile;
    tfSourceFile.setText(sourceFile.getName());
    initProgress();
    if (! sourceFile.canRead()) {
      sourceFileLastModified = -1;
      taResult.setText("file not found: " + sourceFile.getAbsolutePath());
      return;
    }
    sourceFileLastModified = sourceFile.lastModified();
    tfSourceStats.setText("file size: " + FormatUtil.formatSize(sourceFile.length()));
    progress.setMinimum(0);
    progress.setLongMaximum(sourceFile.length());
    updateBtRunState();
  }

  private final Runnable runnableReloadSourceFile = SwingAdapter.getRunnable(this, "reloadSourceFile");
  public void reloadSourceFile() {
    setResultText("");
    selectSourceFile(sourceFile);
  }

  private File outputDir = null;
  public void btOutputDirActionPerformed(ActionEvent ae) {
    JFileChooser fc = new JFileChooser((outputDir == null) ? currentDir : outputDir.getParentFile());
    fc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
    int ret = fc.showOpenDialog(frame);
    if (ret == JFileChooser.APPROVE_OPTION) {
      selectOutputDir(fc.getSelectedFile());
    }
  }

  private void selectOutputDir(File output) {
    outputDir = output;
    tfOutputDir.setText(outputDir.getAbsolutePath());
    initProgress();
    updateBtRunState();
  }

  private void updateBtRunState() {
    boolean enabled = (ld != null) && (logtypeDescription != null) && (sourceFile != null) && (outputDir != null);
    btRun.setEnabled(enabled);
    btBatch.setEnabled(enabled);
  }

  private String resultText = "";
  public void setResultText() {
    taResult.setText(resultText);
  }
  private final Runnable runnableSetResultText = SwingAdapter.getRunnable(this, "setResultText");
  /**
   * set the result text, taking into account if we are on the event dispatch thread or not.
   * @param text the text to display
   */
  private void setResultText(String text) {
    this.resultText = text;
    if (SwingUtilities.isEventDispatchThread()) {
      taResult.setText(resultText);
    } else {
      SwingUtilities.invokeLater(runnableSetResultText);
    }
  }

  private long progressValue = 0;
  private int progressCount = 0;
  private String progressText = null;
  private final Runnable runnableUpdateProgress = SwingAdapter.getRunnable(this, "updateProgress");
  public void updateProgress() {
    if (progressText == null) {
      progress.setValue(progress.getMaximum());
      progress.setString("processed " + progressValue + " logevents");
    } else {
      progress.setLongValue(progressValue);
      progress.setString(progressCount + " - " + progressText);
    }
  }
  private void initProgress() {
    progress.setString("0%");
    progress.setValue(0);
  }

  private final Runnable runnableRunLogDistillation = SwingAdapter.getRunnable(this, "runLogDistillation");
  public void runLogDistillation() {
    try {
      LogEvent.Factory factory = logtypeDescription.newFactory(new InputStreamReader(new FileInputStream(sourceFile)),
          sourceFile.toURL().toString());
      ld.getOutput().setDirectory(outputDir.getAbsolutePath());
      ld.getOutput().setContent("interactive run");
      LogDistillation distillation = new LogDistillation(ld);
      distillation.begin();
      LogEvent le;
      long lastProgress = System.currentTimeMillis();
      while ((le = factory.nextEvent()) != null) {
        distillation.processLogEvent(le);
        long time = System.currentTimeMillis();
        if ((time - lastProgress) > 100) {
          progressValue = distillation.getEventBytes();
          progressCount = distillation.getEventCount();
          progressText = le.getTimestamp();
          SwingUtilities.invokeLater(runnableUpdateProgress);
          lastProgress = time;
        }
      }
      distillation.end();
      progressValue = distillation.getEventCount();
      progressText = null;
      SwingUtilities.invokeLater(runnableUpdateProgress);

      StringWriter sw = new StringWriter();
      new TextReport().report(distillation, sw);
      setResultText(sw.toString());
    } catch (RuntimeException re) {
      ld = null;
      displayException(re);
    } catch (IOException ioe) {
      ld = null;
      displayException(ioe);
    } catch (ParseException pe) {
      ld = null;
      displayException(pe);
    }
    setAllButtonsEnabled(true);
    reloadConfigurationFileEnabled = true;
  }

  public void btRunActionPerformed(ActionEvent ae) {
    reloadConfigurationFileEnabled = false;
    setAllButtonsEnabled(false);
    taResult.setText("");
    new Thread(runnableRunLogDistillation).start();
  }

  public void btBatchActionPerformed(ActionEvent ae) {
    taResult.setText("");
    initProgress();
    try {
      String text = IOUtils.toString(MainPanel.class.getResourceAsStream("reference-build.xml"), "UTF-8");
      Map properties = new HashMap();
      properties.put("id", ld.getId());
      properties.put("configurationFile", configurationFile.getPath());
      properties.put("sourceDir", sourceFile.getAbsoluteFile().getParentFile().getPath());
      properties.put("sourceFile", sourceFile.getName() + "*");
      properties.put("destDir", outputDir.getPath());
      PropertiesReplacer repl = new PropertiesReplacer(properties, "#(", ")");
      taResult.setText(repl.replaceProperties(text));
    } catch (Exception e) {
      displayException(e);
    }
  }

  public void setAllButtonsEnabled(boolean enable) {
    btBatch.setEnabled(enable);
    btOutputDir.setEnabled(enable);
    btRun.setEnabled(enable);
    btSourceFile.setEnabled(enable);
    if (enable) {
      updateBtRunState();
    }
  }

  /**
   * Reload task: checks every second if configuration file or source file has changed.
   */
  public void run() {
    while (true) {
      try {
        Thread.sleep(1000);
      } catch (InterruptedException ie) {
        ie.printStackTrace();
      }
      if (reloadConfigurationFileEnabled && (configurationFile != null) && (configurationFileLastModified != configurationFile.lastModified())) {
        try {
          SwingUtilities.invokeAndWait(runnableReloadConfigurationFile);
        } catch (InvocationTargetException ite) {
          displayException(ite);
        } catch (InterruptedException ie) {
          displayException(ie);
        }
      }
      if ((sourceFile != null) && (sourceFileLastModified != sourceFile.lastModified())) {
        try {
          SwingUtilities.invokeAndWait(runnableReloadSourceFile);
        } catch (InvocationTargetException ite) {
          displayException(ite);
        } catch (InterruptedException ie) {
          displayException(ie);
        }
      }
    }
  }
}
