/*
 *  net/sf/logdistiller/gui/SimpleContainerLayout.java
 *  Copyright (C) 2004-2007 Herv BOUTEMY
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package net.sf.logdistiller.gui;

import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.LayoutManager2;
import java.util.HashMap;
import java.util.Map;

import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

/**
 * This class is a simplified version of FormLayoutMaker's equivalent class, in a dream to avoid need for Java 1.4 at
 * runtime.
 */
public class SimpleContainerLayout implements LayoutManager2 {
  private final Map componentConstraints = new HashMap();
  private final Map componentsToNames = new HashMap();
  private final FormLayout formLayout;
  private final String name;

  public SimpleContainerLayout(String name, String columnSpecs, String rowSpecs) {
    this.name = name;
    formLayout = new FormLayout(columnSpecs, rowSpecs);
  }

  public String getName() {
    return this.name;
  }

  public void addCellConstraints(String name, CellConstraints constraints) {
    componentConstraints.put(name, constraints);
  }

  // interface for LayoutManager2
  public void addLayoutComponent(String name, Component comp) {
    throw new RuntimeException("This method should not be called.  Call addLayoutComponent(Component, Object) instead");
  }

  public float getLayoutAlignmentX(Container target) {
    return formLayout.getLayoutAlignmentX(target);
  }

  public float getLayoutAlignmentY(Container target) {
    return formLayout.getLayoutAlignmentY(target);
  }

  public void invalidateLayout(Container target) {
    formLayout.invalidateLayout(target);
  }

  public void layoutContainer(Container parent) {
    formLayout.layoutContainer(parent);
  }

  public Dimension maximumLayoutSize(Container target) {
    return formLayout.maximumLayoutSize(target);
  }

  public Dimension minimumLayoutSize(Container parent) {
    return formLayout.minimumLayoutSize(parent);
  }

  public Dimension preferredLayoutSize(Container parent) {
    return formLayout.preferredLayoutSize(parent);
  }

  public void removeLayoutComponent(Component comp) {
    String componentName = (String)componentsToNames.get(comp);
    componentsToNames.remove(comp);
    componentConstraints.remove(componentName);
    formLayout.removeLayoutComponent(comp);
  }

  public void addLayoutComponent(Component comp, Object constraints) {
    String componentName = (String)constraints;
    CellConstraints cellConstraints = (CellConstraints)componentConstraints.get(componentName);
    if (cellConstraints == null) {
      System.err.println("Warning: " + componentName + " was added without constraints");
      cellConstraints = new CellConstraints();
      componentConstraints.put(componentName, cellConstraints);
      comp.setVisible(false);
    }
    componentsToNames.put(comp, componentName);
    formLayout.addLayoutComponent(comp, cellConstraints);
  }
}
