/*
 *  net/sf/logdistiller/gui/SimpleLayoutConstraintsManager.java
 *  Copyright (C) 2004-2007 Herv BOUTEMY
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package net.sf.logdistiller.gui;

import java.awt.Insets;
import java.io.InputStream;
import java.util.*;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import com.jgoodies.forms.layout.CellConstraints;

import net.sf.logdistiller.xml.DOMUtils;

/**
 * This class is a simplified version of FormLayoutMaker's equivalent class, in a dream to avoid need for Java 1.4 at
 * runtime.
 */
public class SimpleLayoutConstraintsManager {
  private final Map layouts = new HashMap();

  private SimpleLayoutConstraintsManager() {
  }

  public SimpleContainerLayout getLayout(String name) {
    return (SimpleContainerLayout)layouts.get(name);
  }

  public void addLayout(SimpleContainerLayout containerLayout) {
    layouts.put(containerLayout.getName(), containerLayout);
  }

  public static final String DEFAULT = "default";
  private final static Map CELL_CONSTRAINTS = new HashMap() {
    {
      put("default", CellConstraints.DEFAULT);
      put("fill", CellConstraints.FILL);
      put("center", CellConstraints.CENTER);
      put("left", CellConstraints.LEFT);
      put("right", CellConstraints.RIGHT);
      put("top", CellConstraints.TOP);
      put("bottom", CellConstraints.BOTTOM);
    }
  };

  public static CellConstraints.Alignment getAlignment(String value) {
    CellConstraints.Alignment alignment = (CellConstraints.Alignment)CELL_CONSTRAINTS.get((value == null) ? DEFAULT : value);
    if (alignment == null) {
      throw new RuntimeException("Invalid alignment '" + value + "'");
    }
    return alignment;
  }

  public static SimpleLayoutConstraintsManager getLayoutConstraintsManager(InputStream stream) {
    try {
      DocumentBuilder documentBuilder = DocumentBuilderFactory.newInstance().newDocumentBuilder();
      Document dataDocument = documentBuilder.parse(stream);
      Element root = (Element)dataDocument.getDocumentElement();
      return getLayoutConstraintsManager(root);
    } catch (Exception e) {
      throw new RuntimeException("Unable to create DocumentBuilder", e);
    }
  }

  public static SimpleLayoutConstraintsManager getLayoutConstraintsManager(Element containersNode) {
    if (! containersNode.getNodeName().equals("containers")) {
      throw new RuntimeException("Expected a node named 'containers', found '" + containersNode.getNodeName() + "'");
    }
    SimpleLayoutConstraintsManager layoutConstraintsManager = new SimpleLayoutConstraintsManager();
    Iterator iter = DOMUtils.getChildElementsByTagName(containersNode, "container");
    while (iter.hasNext()) {
      Element containerNode = (Element)iter.next();
      String containerName = containerNode.getAttribute("name");
      String columnSpecs = containerNode.getAttribute("columnSpecs");
      String rowSpecs = containerNode.getAttribute("rowSpecs");
      SimpleContainerLayout containerLayout = new SimpleContainerLayout(containerName, columnSpecs, rowSpecs);

      Iterator iterConstraints = DOMUtils.getChildElementsByTagName(containerNode, "cellconstraints");
      while (iterConstraints.hasNext()) {
        Element constraintsNode = (Element)iterConstraints.next();
        String name = constraintsNode.getAttribute("name");
        CellConstraints.Alignment horizontalAlignment = getAlignment(constraintsNode.getAttribute("horizontalAlignment"));
        CellConstraints.Alignment verticalAlignment = getAlignment(constraintsNode.getAttribute("verticalAlignment"));
        int gridX = getIntValue(constraintsNode, "gridX", 1);
        int gridY = getIntValue(constraintsNode, "gridY", 1);
        int gridWidth = getIntValue(constraintsNode, "gridWidth", 1);
        int gridHeight = getIntValue(constraintsNode, "gridHeight", 1);
        int topInset = getIntValue(constraintsNode, "topInset", 0);
        int bottomInset = getIntValue(constraintsNode, "bottomInset", 0);
        int rightInset = getIntValue(constraintsNode, "rightInset", 0);
        int leftInset = getIntValue(constraintsNode, "leftInset", 0);

        CellConstraints constraints = new CellConstraints (gridX, gridY, gridWidth, gridHeight, horizontalAlignment, verticalAlignment, new Insets (topInset, leftInset, bottomInset, rightInset));
        containerLayout.addCellConstraints(name, constraints);
      }
      layoutConstraintsManager.addLayout(containerLayout);
    }
    return layoutConstraintsManager;
  }

  private static int getIntValue(Element elmt, String name, int defaultValue) {
    String value = elmt.getAttribute(name);
    return (value == null) ? defaultValue : Integer.parseInt(value);
  }
}
