/*
 *  net/sf/logdistiller/gui/ListenerAdapter.java
 *  Copyright (C) 2004-2007 Herv BOUTEMY
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package net.sf.logdistiller.gui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.lang.reflect.*;
import java.awt.event.ItemListener;
import java.awt.event.ItemEvent;

public class SwingAdapter {
  private SwingAdapter() {
  }

  public static ActionListener getActionListener(Object obj, String methodName) {
    try {
      Class[] paramTypes = { ActionEvent.class };
      Method method = obj.getClass().getMethod(methodName, paramTypes);
      return new ActionAdapter(obj, method);
    } catch (NoSuchMethodException nsme) {
      throw new RuntimeException("method 'public void " + methodName + "(ActionEvent ae)' not found in " +
                                         obj.getClass().getName(), nsme);
    }
  }

  public static Runnable getRunnable(Object obj, String methodName) {
    try {
      Class[] paramTypes = { };
      Method method = obj.getClass().getMethod(methodName, paramTypes);
      return new RunnableAdapter(obj, method);
    } catch (NoSuchMethodException nsme) {
      throw new RuntimeException("method 'public void " + methodName + "()' not found in " +
                                         obj.getClass().getName(), nsme);
    }
  }

  public static ItemListener getItemListener(Object obj, String methodName) {
    try {
      Class[] paramTypes = { ItemEvent.class };
      Method method = obj.getClass().getMethod(methodName, paramTypes);
      return new ItemAdapter(obj, method);
    } catch (NoSuchMethodException nsme) {
      throw new RuntimeException("method 'public void " + methodName + "(ItemEvent ie)' not found in " +
                                         obj.getClass().getName(), nsme);
    }
  }

  private static class ActionAdapter implements ActionListener {
    private final Object target;
    private final Method method;
    public ActionAdapter(Object target, Method method) {
      this.target = target;
      this.method = method;
    }

    public void actionPerformed(ActionEvent ae) {
      try {
        method.invoke(target, new Object[] {ae});
      } catch (InvocationTargetException ite) {
        throw new RunException(ite);
      } catch (IllegalArgumentException iae) {
        throw new RuntimeException("IllegalArgumentException invoking method '" + method.getName() + "'", iae);
      } catch (IllegalAccessException iae) {
        throw new RuntimeException("IllegalAccessException invoking method '" + method.getName() + "'", iae);
      }
    }
  }

  private static class RunnableAdapter implements Runnable {
    private final Object target;
    private final Method method;
    public RunnableAdapter(Object target, Method method) {
      this.target = target;
      this.method = method;
    }

    public void run() {
      try {
        method.invoke(target, new Object[] { });
      } catch (InvocationTargetException ite) {
        throw new RunException(ite);
      } catch (IllegalArgumentException iae) {
        throw new RuntimeException("IllegalArgumentException invoking method '" + method.getName() + "'", iae);
      } catch (IllegalAccessException iae) {
        throw new RuntimeException("IllegalAccessException invoking method '" + method.getName() + "'", iae);
      }
    }
  }

  private static class ItemAdapter implements ItemListener {
    private final Object target;
    private final Method method;
    public ItemAdapter(Object target, Method method) {
      this.target = target;
      this.method = method;
    }

    public void itemStateChanged(ItemEvent ie) {
      try {
        method.invoke(target, new Object[] {ie});
      } catch (InvocationTargetException ite) {
        throw new RunException(ite);
      } catch (IllegalArgumentException iae) {
        throw new RuntimeException("IllegalArgumentException invoking method '" + method.getName() + "'", iae);
      } catch (IllegalAccessException iae) {
        throw new RuntimeException("IllegalAccessException invoking method '" + method.getName() + "'", iae);
      }
    }
  }

  public static class RunException extends RuntimeException {
    private static final long serialVersionUID = 4688792600015649872L;

    private RunException(Exception nestedException) {
      super(nestedException);
    }
  }
}
