/*
 *  net/sf/logdistiller/logtypes/SimpleLineLogEvent.java
 *  Copyright (C) 2004-2007 Herv� BOUTEMY
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package net.sf.logdistiller.logtypes;

import java.io.*;
import java.text.ParseException;

import net.sf.logdistiller.LogEvent;
import net.sf.logdistiller.LogType;

/**
 * Generic log event for simple files where one line is one logevent, with an unique attribute named <code>text</code>.
 * By default, the classification rules generated by the GUI for this type of logs won't be able to sort events, as the
 * log event structure is too generic.
 */
public class SimpleLineLogEvent extends LogEvent {
  public final static String ID = "simple";
  public final String text;

  public final static LogType LOGTYPE = new LogType.Basic(ID);
  private final static String[] ATTRIBUTE_NAMES = {
    "logSource",
    "text"
  };
  public final static LogType.Description DESCRIPTION = new Description((LogType.Basic)LOGTYPE, ATTRIBUTE_NAMES);

  public SimpleLineLogEvent(Factory factory, String rawLog) throws ParseException {
    super(factory, rawLog);
    this.text = rawLog;
    setAttributes(new String[] { factory.getLogSource(), text });
  }

  private static class Description extends LogType.Description {
    public Description(LogType.Basic logtype, String[] attributeNames) {
      super(logtype, attributeNames);
      logtype.setDescription(this);
    }

    public LogEvent.Factory newFactory(Reader reader, String logSource) throws IOException {
      return new Factory(this, reader, logSource);
    }

    public int getTimestampAttribute() {
      return -1;
    }
  }

  private static class Factory extends LogEvent.Factory {
    private final LineNumberReader reader;

    public Factory(Description description, Reader reader, String logSource) throws FileNotFoundException {
      super(description, logSource);
      this.reader = new LineNumberReader(reader);
    }

    protected LogEvent readNextEvent() throws IOException, ParseException {
      String curLine = reader.readLine();
      if (curLine == null) {
        // EOF
        return null;
      }
      return new SimpleLineLogEvent(this, curLine);
    }
  }
}
