/*
 *  net/sf/logdistiller/util/ExtensionHelper.java
 *  Copyright (C) 2004-2007 Herv BOUTEMY
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package net.sf.logdistiller.util;

import java.io.IOException;
import java.net.URL;
import java.util.*;


/**
 * Helper to load extensions for <b>LogDistiller</b>. Extensions are declared in <code>logdistiller.properties</code>
 * resource file, as properties. Each extensible part can be declared as a property which key value specifies the
 * part extended (<code>logtypes</code>, <code>plugins</code>, <code>publishers</code> or <code>reportformats</code>),
 * the value is the full class name (with package) of the extension.
 */
public class ExtensionHelper {
  private static List extensionProperties;

  public static List findExtensions(String type) {
    if (extensionProperties == null) {
      extensionProperties = findExtensions();
    }
    List extensions = new ArrayList();
    Iterator iter = extensionProperties.iterator();
    while (iter.hasNext()) {
      Properties prop = (Properties)iter.next();
      String extension = prop.getProperty(type);
      if (extension != null) {
        extensions.add(extension);
      }
    }
    return extensions;
  }

  private static List findExtensions() {
    URL url = null;
    try {
      List extensions = new ArrayList();
      Enumeration res = ExtensionHelper.class.getClassLoader().getResources("logdistiller.properties");
      while (res.hasMoreElements()) {
        url = (URL) res.nextElement();
        Properties prop = new Properties();
        prop.load(url.openStream());
        extensions.add(prop);
      }
      return extensions;
    } catch (IOException ioe) {
      String msg = (url == null) ? "unable to list resources logdistiller.properties"
         : "unable to load resource " + url.toExternalForm();
      throw new RuntimeException(msg, ioe);
    }
  }
}
