/*
 *  net/sf/logdistiller/util/PropertiesReplacer.java
 *  Copyright (C) 2004-2007 Herv BOUTEMY
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package net.sf.logdistiller.util;

import java.util.Map;

/**
 * A small utility to replace properties within strings (syntax: <code>${<i>name</i>}</code>).
 */
public class PropertiesReplacer {
  private final String beginMark;
  private final String endMark;
  private final Map properties;

  public PropertiesReplacer(Map properties, String beginMark, String endMark) {
    this.properties = properties;
    this.beginMark = beginMark;
    this.endMark = endMark;
  }

  public PropertiesReplacer(Map properties) {
    this(properties, "${", "}");
  }

  /**
   * Replace <code>${<i>name</i>}</code> by the value of the property.
   * @param source String the string to interpret (or <code>null</code>)
   * @return String the transformed string
   * @throws IllegalArgumentException if the source string references an inexistent property
   */
  public String replaceProperties(String source) {
    if (source == null) {
      return null;
    }
    int index = 0;
    int nextBegin, nextEnd;
    StringBuffer buff = new StringBuffer();
    while (((nextBegin = source.indexOf(beginMark, index)) >= 0) &&
           ((nextEnd = source.indexOf(endMark, nextBegin)) >= 0)) {
      String name = source.substring(nextBegin + 2, nextEnd);
      String value = (String)properties.get(name);
      if (value == null) {
        throw new IllegalArgumentException("unknown property '" + name + "'");
      }
      buff.append(source.substring(index, nextBegin));
      buff.append(value);
      index = nextEnd + 1;
    }
    if (index == 0) {
      // little optimization...
      return source;
    }
    buff.append(source.substring(index));
    return buff.toString();
  }
}
