/*
 *  net/sf/logdistiller/xml/LogDistillerEntityResolver.java
 *  Copyright (C) 2004-2007 Herv BOUTEMY
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package net.sf.logdistiller.xml;

import java.io.InputStream;
import java.util.HashSet;
import java.util.Set;
import org.xml.sax.EntityResolver;
import org.xml.sax.InputSource;

public class LogDistillerEntityResolver implements EntityResolver {
  public final static String LATEST_DTD = "logdistiller-1_3.dtd";
  public final static String LATEST_DTD_URL = "http://logdistiller.sourceforge.net/dtd/" + LATEST_DTD;
  private final static Set DTDS = new HashSet() {
    {
      add("logdistiller-1_0.dtd");
      add("logdistiller-1_1.dtd");
      add("logdistiller-1_2.dtd");
      add("logdistiller-1_3.dtd");
    }
  };
  private final static int LEN = 20; // length of dtd string...
  private String currentDtd;

  public InputSource resolveEntity (String publicId, String systemId) {
    if (systemId.length() < LEN) {
      return null;
    }
    String dtd = systemId.substring(systemId.length() - LEN);
    if (DTDS.contains(dtd)) {
      currentDtd = dtd;
      InputStream in = LogDistillerEntityResolver.class.getResourceAsStream(dtd);
      if (in == null) {
        throw new RuntimeException("unable to load DTD " + dtd);
      } else {
        return new InputSource(in);
      }
    } else {
      return null;
    }
  }

  public String getCurrentDtd() {
    return currentDtd;
  }
}
